/*
    Copyright (C) 1999 by Joe Pasqua <joe@NoRoomAtTheInn.org>.
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions
    are met:
    1. Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.


    THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

import java.util.Hashtable;
import java.util.Vector;

class TimeVal extends Value {
    private int hours;      // 0-23
    private int minutes;    // 0-59
    private int offset = 0;
    private int hiByte = 0;

    void init(Type t, String value)
            throws ParseException
    {
        int[]   parts;
        String  delims = "-:+";

        super.init(t, value);

        try {
            parts = Value.parts(value, delims, 0, 23, 0, 59);
            if (parts[2] != 1) {    // It was - or +
                if (parts[0] == 17 /*'R'*/)
                    hiByte = 0x40;
                else if (parts[0] == 18 /*'S'*/)
                    hiByte = 0x80;
                else throw new ParseException(
                    "Use 'R' for sunRise, 'S' for sunSet: " + value);
                if (parts[1] > 30)
                    throw new ParseException(
                        "Offset must be between -30..30: " + value);
                offset = parts[1] * (parts[2]-1);

            }
            else {
                hours = parts[0];
                minutes = parts[1];
            }
        } catch (ParseException e) {
            throw new ParseException(
                    "Bad value for " + t + ": " + value + " (" +
                    e.getMessage() + ")");
        }

    }

    public int getEncodedTime() {
        if (hiByte == 0)
            return (hours * 100) + minutes;
        return (hiByte << 8)+ (30+offset);
    }

    public String toString() {
        if (hiByte == 0)
            return ("\"" + hours + ":" +
                ((minutes < 10) ? "0" : "") + minutes + "\"");
        return (hiByte == 0x40 ? "R" : "S") +
                (offset < 0 ? ("-" + -offset) : ("+" + offset));
    }

}
